@@ -13,9 +13,7 @@ from __future__ import absolute_import, division, print_function, with_statement
 import os
 from .util import tmux, TmuxRelationalObject
 from .session import Session
-from .formats import SESSION_FORMATS, CLIENT_FORMATS
-
-from . import log
+from . import formats
 import logging
 
 logger = logging.getLogger(__name__)
@@ -43,11 +41,19 @@ class Server(TmuxRelationalObject):
     config_file = None
     childIdAttribute = 'session_id'
 
-    def __init__(self, **kwargs):
-        self._sessions = list()
-        self.children = self._sessions
+    def __init__(self, socket_name=None, socket_path=None, config_file=None,
+                 **kwargs):
+        self._windows = []
+        self._panes = []
 
-        self._TMUX = {}
+        if socket_name:
+            self.socket_name = socket_name
+
+        if socket_path:
+            self.socket_path = socket_path
+
+        if config_file:
+            self.config_file = config_file
 
     def tmux(self, *args, **kwargs):
         args = list(args)
@@ -57,83 +63,197 @@ class Server(TmuxRelationalObject):
             args.insert(0, '-S{}'.format(self.socket_path))
         if self.config_file:
             args.insert(0, '-f{}'.format(self.config_file))
+
         return tmux(*args, **kwargs)
 
-    def list_sessions(self):
+    def __list_sessions(self):
         '''
-        Return a list of :class:`Session` from tmux server.
+        compatibility wrapping for ``$ tmux list-sessions``.
 
-        ``$ tmux list-sessions``
+        :rtype: stdout or stderr of tmux proc
         '''
-        formats = SESSION_FORMATS
-        tmux_formats = ['#{%s}' % format for format in formats]
-        sessions = self.tmux(
-            'list-sessions',
+        sformats = formats.SESSION_FORMATS
+        tmux_formats = ['#{%s}' % f for f in sformats]
+
+        tmux_args = (
             '-F%s' % '\t'.join(tmux_formats),   # output
-        ).stdout
+        )
+
+        proc = self.tmux(
+            'list-sessions',
+            *tmux_args
+        )
+
+        if proc.stderr:
+            if 'unknown option -- F' in proc.stderr[0]:
+                proc = self.tmux(
+                    'list-sessions',
+                )
+
+                if proc.stderr:
+                    raise Exception(proc.stderr)
+                else:
+                    return 'hi'
+            else:
+                raise Exception(proc.stderr)
+        else:
+            session_info = proc.stdout[0]
+
+        if proc.stderr:
+            raise Exception(sessions.stderr)
+
+        return proc.stdout
+
+    def _list_sessions(self):
+        '''
+        Return a list of session information ``tmux(1)`` for the sessions
+
+        :rtype: :py:obj:`list` of :py:obj:`dict`
+        '''
+
+        sformats = formats.SESSION_FORMATS
+        tmux_formats = ['#{%s}' % format for format in sformats]
+        sessions = self.__list_sessions()
 
         # combine format keys with values returned from ``tmux list-windows``
         sessions = [dict(zip(
-            formats, session.split('\t'))) for session in sessions]
+            sformats, session.split('\t'))) for session in sessions]
 
         # clear up empty dict
-        new_sessions = [
-            dict((k, v) for k, v in session.iteritems() if v) for session in sessions
+        sessions = [
+            dict((k, v) for k, v in session.items() if v) for session in sessions
         ]
 
-        if not self._sessions:
-            for session in new_sessions:
-                logger.debug('adding session_id %s' % (session['session_id']))
-                new_session = Session(server=self, **session)
-                self._sessions.append(new_session)
-            return self._sessions
+        return sessions
 
-        new = {session['session_id']: session for session in new_sessions}
-        old = {session.get(
-            'session_id'): session for session in self._sessions}
+    @property
+    def _sessions(self):
+        return self._list_sessions()
 
-        created = set(new.keys()) - set(old.keys()) or ()
-        deleted = set(old.keys()) - set(new.keys()) or ()
-        intersect = set(new.keys()).intersection(set(old.keys()))
+    def list_sessions(self):
+        '''
+        Return a list of :class:`Session` from the ``tmux(1)`` session.
 
-        diff = {id: dict(set(new[id].items()) - set(old[id].items()))
-                for id in intersect}
+        :rtype: :py:obj:`list` of :class:`Session`
+        '''
+        return [
+            Session(server=self, **s) for s in self._sessions
+        ]
 
-        intersect = set(k for k, v in diff.iteritems() if v) or ()
-        diff = dict((k, v) for k, v in diff.iteritems() if v) or ()
+    @property
+    def sessions(self):
+        return self.list_sessions()
+    children = sessions
 
-        if diff or created or deleted:
-            log_diff = "sync sessions for server:\n"
-        else:
-            log_diff = None
-        if diff and intersect:
-            log_diff += "diff %s for %s" % (diff, intersect)
-        if created:
-            log_diff += "created %s" % created
-        if deleted:
-            log_diff += "deleted %s" % deleted
-        if log_diff:
-            logger.debug(log_diff)
+    def __list_windows(self):
+        '''
+        Return dict of ``tmux(1) list-windows`` values.
+        '''
 
-        for s in self._sessions:
-            # remove session objects if deleted or out of session
-            if s.get('session_id') in deleted:
-                logger.debug("removing %s" % s)
-                self._sessions.remove(s)
+        wformats = ['session_name', 'session_id'] + formats.WINDOW_FORMATS
+        tmux_formats = ['#{%s}' % format for format in wformats]
 
-            if s.get('session_id') in intersect and s.get('session_id') in diff:
-                logger.debug('updating session_id %s session_name %s' % (
-                    s.get('session_id'), s.get('session_name')))
-                s.update(diff[s.get('session_id')])
+        windows = self.tmux(
+            'list-windows',                     # ``tmux list-windows``
+            '-a',
+            '-F%s' % '\t'.join(tmux_formats),   # output
+        )
 
-        # create session objects for non-existant session_id's
-        for session in [new[session_id] for session_id in created]:
-            logger.debug('new session %s' % session['session_id'])
-            new_session = Session(server=self, **session)
-            self._sessions.append(new_session)
+        if windows.stderr:
+            raise Exception(windows.stderr)
 
-        return self._sessions
-    list_children = list_sessions
+        return windows.stdout
+
+    def _list_windows(self):
+        ''' take the outpout of _list_windows from shell and put it into
+        a list of dicts'''
+
+        wformats = ['session_name', 'session_id'] + formats.WINDOW_FORMATS
+
+        windows = self.__list_windows()
+
+        # combine format keys with values returned from ``tmux list-windows``
+        windows = [dict(zip(
+            wformats, window.split('\t'))) for window in windows]
+
+        # clear up empty dict
+        windows = [
+            dict((k, v) for k, v in window.items() if v) for window in windows
+        ]
+
+        # tmux < 1.8 doesn't have window_id, use window_name
+        for w in windows:
+            if not 'window_id' in w:
+                w['window_id'] = w['window_name']
+
+        if self._windows:
+            # http://stackoverflow.com/a/14465359
+            self._windows[:] = []
+
+        self._windows.extend(windows)
+
+        return self._windows
+
+    def _update_windows(self):
+        self._list_windows()
+        return self
+
+    def __list_panes(self):
+        '''Return list of :class:`Pane` for the window.
+
+        :rtype: list of :class:`Pane`
+        '''
+        pformats = ['session_name', 'session_id',
+                    'window_index', 'window_id', 'window_name'] + formats.PANE_FORMATS
+        tmux_formats = ['#{%s}\t' % f for f in pformats]
+
+        panes = self.tmux(
+            'list-panes',
+            #'-t%s:%s' % (self.get('session_name'), self.get('window_id')),
+            '-a',
+            '-F%s' % ''.join(tmux_formats),     # output
+        )
+
+        if panes.stderr:
+            raise Exception(panes.stderr)
+
+        return panes.stdout
+
+    def _list_panes(self):
+        ''' take the outpout of _list_panes from shell and put it into
+        a list of dicts'''
+
+        pformats = ['session_name', 'session_id',
+                    'window_index', 'window_id', 'window_name'] + formats.PANE_FORMATS
+
+        panes = self.__list_panes()
+
+        # combine format keys with values returned from ``tmux list-panes``
+        panes = [dict(zip(
+            pformats, window.split('\t'))) for window in panes]
+
+        # clear up empty dict
+        panes = [
+            dict((k, v) for k, v in window.items() if v) for window in panes
+        ]
+
+        # tmux < 1.8 doesn't have window_id, use window_name
+        for p in panes:
+            if not 'window_id' in p:
+                p['window_id'] = p['window_name']
+
+
+        if self._panes:
+            # http://stackoverflow.com/a/14465359
+            self._panes[:] = []
+
+        self._panes.extend(panes)
+
+        return self._panes
+
+    def _update_panes(self):
+        self._list_panes()
+        return self
 
     def list_clients(self):
         '''
@@ -141,8 +261,8 @@ class Server(TmuxRelationalObject):
 
         ``$ tmux list-clients``
         '''
-        formats = CLIENT_FORMATS
-        tmux_formats = ['#{%s}' % format for format in formats]
+        cformats = CLIENT_FORMATS
+        tmux_formats = ['#{%s}' % format for format in cformats]
         # import ipdb
         # ipdb.set_trace()
         clients = self.tmux(
@@ -152,11 +272,11 @@ class Server(TmuxRelationalObject):
 
         # combine format keys with values returned from ``tmux list-windows``
         clients = [dict(zip(
-            formats, client.split('\t'))) for client in clients]
+            cformats, client.split('\t'))) for client in clients]
 
         # clear up empty dict
         new_clients = [
-            dict((k, v) for k, v in client.iteritems() if v) for client in clients
+            dict((k, v) for k, v in client.items() if v) for client in clients
         ]
 
         if not self._clients:
@@ -200,28 +320,12 @@ class Server(TmuxRelationalObject):
 
         return self._clients
 
-    def server_exists(self):
-        '''server is on and exists
-
-        '''
-
-        try:
-            self.tmux('list-clients')
-            self.tmux('list-sessions')
-            return True
-        except Exception:
-            return False
-
     def has_clients(self):
         # are any clients connected to tmux
-        if len(self.tmux('list-clients')) > int(1):
-            return True
-        else:
-            return False
-        # if e.stderr == 'failed to connect to server':
-        #    raise TmuxNotRunning('tmux session not running. please start'
-        #                            'a tmux session in another terminal '
-        #                            'window and continue.')
+        proc = self.tmux('list-clients')
+
+        if proc.stderr:
+            raise Exception(proc.stderr)
 
     def attached_sessions(self):
         '''
@@ -276,12 +380,12 @@ class Server(TmuxRelationalObject):
         :param: target_session: str. note this accepts fnmatch(3). 'asdf' will
                                 kill asdfasd
         '''
-        self.tmux('kill-session', '-t%s' % target_session)
-        self.list_sessions()
+        proc = self.tmux('kill-session', '-t%s' % target_session)
 
-    @property
-    def sessions(self):
-        return self._sessions
+        if proc.stderr:
+            raise Exception(proc.stderr)
+
+        return self
 
     def switch_client(self, target_session):
         '''
@@ -290,11 +394,31 @@ class Server(TmuxRelationalObject):
         :param: target_session: str. name of the session. fnmatch(3) works.
         '''
         # tmux('switch-client', '-t', target_session)
-        self.tmux('switch-client', '-t%s' % target_session)
+        proc = self.tmux('switch-client', '-t%s' % target_session)
+
+        if proc.stderr:
+            raise Exception(proc.stderr)
+
+    def attach_session(self, target_session=None):
+        '''
+        ``$ tmux attach-session`` aka alias: ``$ tmux attach``
+
+        :param: target_session: str. name of the session. fnmatch(3) works.
+        '''
+        # tmux('switch-client', '-t', target_session)
+        tmux_args = tuple()
+        if target_session:
+            tmux_args += ('-t%s' % target_session,)
+
+        proc = self.tmux('attach-session', *tmux_args)
+
+        if proc.stderr:
+            raise Exception(proc.stderr)
 
     def new_session(self,
                     session_name=None,
                     kill_session=False,
+                    attach=False,
                     *args,
                     **kwargs):
         '''
@@ -333,44 +457,50 @@ class Server(TmuxRelationalObject):
         if self.has_session(session_name):
             if kill_session:
                 self.tmux('kill-session', '-t%s' % session_name)
-                logger.error('session %s exists. killed it.' % session_name)
+                logger.info('session %s exists. killed it.' % session_name)
             else:
                 raise TmuxSessionExists(
                     'Session named %s exists' % session_name)
 
         logger.debug('creating session %s' % session_name)
 
-        formats = SESSION_FORMATS
-        tmux_formats = ['#{%s}' % format for format in formats]
+        sformats = formats.SESSION_FORMATS
+        tmux_formats = ['#{%s}' % f for f in sformats]
 
         env = os.environ.get('TMUX')
 
         if env:
             del os.environ['TMUX']
 
-        session_info = self.tmux(
-            'new-session',
-            '-d',  # assume detach = True for now, todo: fix
+        tmux_args = (
             '-s%s' % session_name,
             '-P', '-F%s' % '\t'.join(tmux_formats),   # output
         )
 
-        session_info = session_info.stdout[0]
+        if not attach:
+            tmux_args += ('-d',)
+
+        proc = self.tmux(
+            'new-session',
+            *tmux_args
+        )
+
+        if proc.stderr:
+            raise Exception(proc.stderr)
+
+        session = proc.stdout[0]
 
         if env:
             os.environ['TMUX'] = env
 
         # combine format keys with values returned from ``tmux list-windows``
-        session_info = dict(zip(formats, session_info.split('\t')))
+        session = dict(zip(sformats, session.split('\t')))
 
         # clear up empty dict
-        session_info = dict((k, v) for k, v in session_info.iteritems() if v)
+        session = dict((k, v) for k, v in session.items() if v)
 
-        session = Session(server=self, **session_info)
+        session = Session(server=self, **session)
 
-        # need to be able to get first windows
-        session._windows = session.list_windows()
-
-        self._sessions.append(session)
+        # self._sessions.append(session)
 
         return session
