@@ -27,20 +27,21 @@ def bootstrap():
 
     '''
 
-    session_list = t.list_sessions()
+    # session_list = t.sessions
 
-    assert session_list == t.list_sessions()
+    # assert session_list == t.sessions
 
     # find current sessions prefixed with tmuxp
-    old_test_sessions = [s.get('session_name') for s in session_list
-                        if s.get('session_name').startswith(TEST_SESSION_PREFIX)]
+    # old_test_sessions = [s.get('session_name') for s in session_list
+                        # if
+                        # s.get('session_name').startswith(TEST_SESSION_PREFIX)]
 
-    other_sessions = [s.get('session_name') for s in session_list
-                      if not s.get('session_name').startswith(
-                          TEST_SESSION_PREFIX
-                      )]
+    # other_sessions = [s.get('session_name') for s in session_list
+                      # if not s.get('session_name').startswith(
+                          # TEST_SESSION_PREFIX
+                      # )]
 
-    assert session_list == t.list_sessions()
+    # assert session_list == t.sessions
 
     TEST_SESSION_NAME = TEST_SESSION_PREFIX + str(randint(0, 13370))
     session = t.new_session(
@@ -51,13 +52,17 @@ def bootstrap():
     Make sure that tmuxp can :ref:`test_builder_visually` and switches to the
     newly created session for that testcase.
     '''
-    t.switch_client(session.get('session_id'))
-
-    for old_test_session in old_test_sessions:
-        logger.debug('Old test test session %s found. Killing it.' %
-                      old_test_session)
-        t.kill_session(old_test_session)
+    try:
+        t.switch_client(session.get('session_id'))
+        pass
+    except Exception:
+        # t.attach_session(session.get('session_id'))
+        pass
 
+    # for old_test_session in old_test_sessions:
+        # logger.debug('Old test test session %s found. Killing it.' %
+                      # old_test_session)
+        # t.kill_session(old_test_session)
     assert TEST_SESSION_NAME == session.get('session_name')
 
     return (TEST_SESSION_NAME, session)
@@ -78,8 +83,10 @@ class TmuxTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         try:
+            t.tmux('kill-server')
+
             cls.TEST_SESSION_NAME, cls.session = bootstrap()
-        #except TmuxNoClientsRunning:
+        # except TmuxNoClientsRunning:
         #    logger.error('test: TmuxNoClientsRunning')
         #    cls.TEST_SESSION_NAME, cls.session = bootstrap()
         except Exception as e:
@@ -87,9 +94,10 @@ class TmuxTestCase(unittest.TestCase):
             logger.error(e)
 
             logger.error(traceback.print_exc())
-            #raise Exception(e)
+            # raise Exception(e)
         return
 
     @classmethod
     def tearDownClass(cls):
+        t.tmux('kill-server')
         pass
