@@ -2,7 +2,7 @@
 from __future__ import absolute_import, division, print_function, with_statement
 
 import unittest
-from .. import Pane
+from .. import Pane, Window, Server
 from .helpers import TmuxTestCase
 from . import t
 
@@ -15,7 +15,7 @@ logger = logging.getLogger(__name__)
 class SelectTest(TmuxTestCase):
 
     def test_select_window(self):
-        window_count = len(self.session.list_windows())
+        window_count = len(self.session._windows)
         # to do, get option for   base-index from tmux
         # for now hoever, let's get the index from the first window.
         self.assertEqual(window_count, 1)
@@ -25,7 +25,6 @@ class SelectTest(TmuxTestCase):
 
         window = self.session.new_window(window_name='testing 3')
 
-        # logger.error([window._TMUX for window in self.session.list_windows()])
         # self.assertEqual(2,
         # int(self.session.attached_window().get('window_index')))
         self.assertEqual(int(window_base_index) + 1, int(
@@ -39,7 +38,7 @@ class SelectTest(TmuxTestCase):
         self.assertEqual(int(window_base_index) + 1, int(
             self.session.attached_window().get('window_index')))
 
-        self.assertEqual(len(self.session.list_windows()), 2)
+        self.assertEqual(len(self.session._windows), 2)
 
 
 class NewTest(TmuxTestCase):
@@ -47,8 +46,13 @@ class NewTest(TmuxTestCase):
     def test_zfresh_window_data(self):
         # self.session.select_window(1)
         current_windows = len(self.session._windows)
-        # logger.error("current panes: %s" %
-        # len(self.session.attached_window()._panes))
+        self.assertIsInstance(self.session.server, Server)
+        # len(self.session.attached_window().panes))
+        for w in self.session.windows:
+            self.assertIsInstance(w, Window)
+        window = self.session.attached_window()
+        self.assertIsInstance(window, Window)
+        pane = window.split_window()
         self.session.attached_window().select_pane(0)
         self.session.attached_pane().send_keys('cd /srv/www/flaskr')
         self.session.attached_window().select_pane(1)
@@ -65,18 +69,30 @@ class NewTest(TmuxTestCase):
         current_windows -= 1
         self.assertEqual(current_windows, len(self.session._windows))
 
+
+class NewTest2(TmuxTestCase):
+
     def test_newest_pane_data(self):
         # self.session.select_window(1)
-        self.session.attached_window().list_panes()
-        self.assertEqual(1, len(self.session.attached_window()._panes))
-        self.session.attached_window().select_layout('even-horizontal')
+        #
+        #
+        window = self.session.new_window(window_name='test', attach=True)
+        self.assertIsInstance(window, Window)
+        self.assertEqual(1, len(window.panes))
+        window.select_layout()
+        window.split_window(attach=True)
+        window.select_layout()
 
-        self.session.attached_window().split_window(attach=True)
-        self.assertEqual(2, len(self.session.attached_window()._panes))
+        self.assertEqual(2, len(window.panes))
         # note: the below used to accept -h, removing because split_window now
         # has attach as its only argument now
-        self.session.attached_window().split_window(attach=True)
-        self.assertEqual(3, len(self.session.attached_window()._panes))
+        window.select_layout()
+        window.split_window(attach=True)
+        self.assertEqual(3, len(window.panes))
+        window.select_layout()
+
+
+class NewTest3(TmuxTestCase):
 
     def test_attached_pane(self):
         '''Window.attached_window() returns active Pane'''
@@ -84,22 +100,16 @@ class NewTest(TmuxTestCase):
         window = self.session.attached_window()  # current window
         self.assertIsInstance(window.attached_pane(), Pane)
 
+
+class NewTest4(TmuxTestCase):
+
     def test_split_window(self):
         '''Window.split_window() splits window, returns new Pane.'''
         window_name = 'test split window'
         window = self.session.new_window(window_name=window_name, attach=True)
-        pane = window.split_window(attach=True)
-        self.assertEqual(2, len(self.session.attached_window()._panes))
+        pane = window.split_window()
+        self.assertEqual(2, len(window.panes))
         self.assertIsInstance(pane, Pane)
-        self.assertEqual(2, len(window._panes))
-
-    def test_window_rename(self):
-        '''Window.rename_window() renames window'''
-        window = self.session.attached_window()
-        window_name_before = window.get('window_name')
-        window_name_after = 'testingdis_winname'
-        window.rename_window(window_name_after)
-        self.assertEqual(window.get('window_name'), window_name_after)
 
 
 class RenameTest(TmuxTestCase):
@@ -109,8 +119,9 @@ class RenameTest(TmuxTestCase):
 
     def test_window_rename(self):
         '''Window.rename_window.rename_window()'''
-
-        window = self.session.new_window(window_name=self.window_name_before)
+        self.session.set_option('automatic-rename', 'off')
+        window = self.session.new_window(
+            window_name=self.window_name_before, attach=True)
 
         self.assertEqual(window, self.session.attached_window())
         self.assertEqual(window.get('window_name'), self.window_name_before)
@@ -121,8 +132,6 @@ class RenameTest(TmuxTestCase):
 
         self.assertEqual(window.get('window_name'), self.window_name_after)
 
-        self.session.list_windows()
-
         window = self.session.attached_window()
 
         self.assertEqual(window.get('window_name'), self.window_name_after)
