@@ -12,9 +12,8 @@ from __future__ import absolute_import, division, print_function, with_statement
 
 import pipes
 from .pane import Pane
-from .formats import PANE_FORMATS
 
-from . import log, util
+from . import util, formats
 import logging
 
 logger = logging.getLogger(__name__)
@@ -29,19 +28,24 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
     childIdAttribute = 'pane_id'
 
     def __init__(self, session=None, **kwargs):
-        self._panes = list()  # list of panes
-        self.children = self._panes
 
         if not session:
-            raise ValueError(
-                "Window requires a Session object by "
-                "specifying session=Session"
-            )
+            raise ValueError('Window requires a Session, session=Session')
+
         self.session = session
         self.server = self.session.server
 
-        self._TMUX = {}
-        self.update(**kwargs)
+        if not 'window_id' in kwargs:
+            if kwargs['window_index']:
+                self._window_id = kwargs['window_index']
+            else:
+                raise ValueError('Window requires a `window_id`')
+
+        if not 'window_index' in kwargs:
+            raise ValueError('Window requires a `window_index`')
+        self._window_index = kwargs['window_index']
+
+        #self.server._update_windows()
 
     def __repr__(self):
         return "%s(%s %s:%s, %s)" % (
@@ -52,6 +56,25 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
             self.session
         )
 
+    @property
+    def _TMUX(self, *args):
+
+        attrs = {
+            'window_index': self._window_index
+        }
+
+        # from https://github.com/serkanyersen/underscore.py
+        def by(val, *args):
+            for key, value in attrs.items():
+                try:
+                    if attrs[key] != val[key]:
+                        return False
+                except KeyError:
+                    return False
+                return True
+
+        return list(filter(by, self.server._windows))[0]
+
     def tmux(self, *args, **kwargs):
         # if '-t' not in kwargs:
         #    kwargs['-t'] = self.get['session_id']
@@ -123,7 +146,8 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
         if process.stderr:
             if isinstance(process.stderr, list) and len(process.stderr) == int(1):
                 process.stderr = process.stderr[0]
-            raise ValueError('tmux set-window-option stderr: %s' % process.stderr)
+            raise ValueError(
+                'tmux set-window-option stderr: %s' % process.stderr)
 
     def show_window_options(self, option=None):
         '''
@@ -148,7 +172,7 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
 
         window_options = dict(window_options)
 
-        for key, value in window_options.iteritems():
+        for key, value in window_options.items():
             if value.isdigit():
                 window_options[key] = int(value)
 
@@ -170,7 +194,8 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
         ).stdout
 
         if window_option:
-            window_option = [tuple(item.split(' ')) for item in window_option][0]
+            window_option = [tuple(item.split(' '))
+                             for item in window_option][0]
         else:
             return None
 
@@ -187,13 +212,13 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
         :param new_name: name of the window
         :type new_name: string
         '''
-        #new_name = pipes.quote(new_name)
+        # new_name = pipes.quote(new_name)
 
         import shlex
         lex = shlex.shlex(new_name)
         lex.escape = ' '
         lex.whitespace_split = False
-        #new_name = '\ '.join(new_name.split())
+        # new_name = '\ '.join(new_name.split())
 
         try:
             self.tmux(
@@ -205,7 +230,7 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
         except Exception as e:
             logger.error(e)
 
-        self.session.list_windows()
+        self.server._update_windows()
 
         return self
 
@@ -219,15 +244,14 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
 
         Todo: make 'up', 'down', 'left', 'right' acceptable ``target_pane``.
         '''
-        #if isinstance(target_pane, basestring) and not ':' not in target_pane or isinstance(target_pane, int):
+        # if isinstance(target_pane, basestring) and not ':' not in target_pane or isinstance(target_pane, int):
         #    target_pane = "%s.%s" % (self.target, target_pane)
 
-        try:
-            self.tmux('select-pane', '-t%s' % target_pane)
-        except Exception:
-            logger.error('pane not found %s %s' % (
-                target_pane, self.list_panes()))
-        self.list_panes()
+        proc = self.tmux('select-pane', '-t%s' % target_pane)
+
+        if proc.stderr:
+            raise Exception(proc.stderr)
+
         return self.attached_pane()
 
     def split_window(self, attach=True):
@@ -255,12 +279,15 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
         :rtype: :class:`Pane`
         '''
 
-        formats = ['session_name', 'session_id',
-                   'window_index', 'window_id'] + PANE_FORMATS
-        tmux_formats = ['#{%s}\t' % format for format in formats]
+        pformats = ['session_name', 'session_id',
+                    'window_index', 'window_id'] + formats.PANE_FORMATS
+        tmux_formats = ['#{%s}\t' % f for f in pformats]
+
+        #'-t%s' % self.attached_pane().get('pane_id'),
+        # 2013-10-18 LOOK AT THIS, rm'd it..
 
         tmux_args = (
-            '-t%s' % self.attached_pane().get('pane_id'),
+            '-t%s' % self.panes[0].get('pane_id'),
             '-P', '-F%s' % ''.join(tmux_formats),     # output
         )
 
@@ -270,18 +297,46 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
         pane = self.tmux(
             'split-window',
             *tmux_args
-        ).stdout[0]
+        )
 
-        # zip and map the results into the dict of formats used above
-        pane = dict(zip(formats, pane.split('\t')))
+        # tmux < 1.7. This is added in 1.7.
+        if pane.stderr:
+            if 'unknown option -- F' in pane.stderr[0]:
+                tmux_args = (
+                    '-t%s' % self.panes[0].get('pane_id'),
+                )
+                if not attach:
+                    tmux_args += ('-d',)
+                proc = self.tmux(
+                    'split-window',
+                    '-P',
+                    *tmux_args
+                )
+                if proc.stderr:
+                    raise Exception(pane.stderr)
 
-        # clear up empty dict
-        pane = dict((k, v) for k, v in pane.iteritems() if v)
-        pane = Pane(window=self, **pane)
-        self._panes.append(pane)
-        self.list_panes()  # refresh all panes in :class:`Window`
+                pane = proc.stdout[0]
+                import re
+                pane_regex = re.compile(r"^(?P<session_name>.*):(?P<window_index>\d*?).(?P<pane_index>\d*?)$")
+                m = re.match(pane_regex, pane)
+                pane = m.groupdict()
 
-        return pane
+                pane['pane_id'] = pane['pane_index']
+
+                #pane = dict(zip(['session_name', 'window_index'], pane[0].split(':')))
+
+                # todo, this needs code to find the newest pane created
+            else:
+                raise Exception(pane.stderr)
+        else:
+            pane = pane.stdout[0]
+
+            pane = dict(zip(pformats, pane.split('\t')))
+
+            # clear up empty dict
+            pane = dict((k, v) for k, v in pane.items() if v)
+
+        return Pane(window=self, **pane)
 
     def attached_pane(self):
         '''
@@ -289,120 +344,40 @@ class Window(util.TmuxMappingObject, util.TmuxRelationalObject):
 
         :rtype: :class:`Pane`
         '''
-        panes = self.list_panes()
-
-        for pane in panes:
+        for pane in self._panes:
             if 'pane_active' in pane:
                 # for now pane_active is a unicode
                 if pane.get('pane_active') == '1':
-                    return pane
+                    # return Pane(window=self, **pane)
+                    return Pane(window=self, **pane)
                 else:
                     continue
 
-        # if the client is not on the window and none is active, assume the
-        # first pane.
-        if panes:
-            return panes[0]
+    def _list_panes(self):
+        panes = self.server._update_panes()._panes
 
-        return False
+        panes = [
+            p for p in panes if p['session_name'] == self.get('session_name')
+        ]
+        panes = [
+            p for p in panes if p['window_id'] == self.get('window_id')
+        ]
+        return panes
 
-    def refresh(self):
-        '''Refresh current :class:`Window` object. Chainable.
-
-        :rtype: :class:`Window`
-        '''
-        self._TMUX = self.server.getById(self['window_id'])._TMUX
-
-        return self
+    @property
+    def _panes(self):
+        return self._list_panes()
 
     def list_panes(self):
         '''Return list of :class:`Pane` for the window.
 
         :rtype: list of :class:`Pane`
         '''
-        formats = ['session_name', 'session_id',
-                   'window_index', 'window_id'] + PANE_FORMATS
-        tmux_formats = ['#{%s}\t' % format for format in formats]
 
-        # if isinstance(self.get('window_id'), basestring):
-        #    window_id =
+        return [Pane(window=self, **pane) for pane in self._panes]
 
-        panes = self.tmux(
-            'list-panes',
-            '-t%s:%s' % (self.get('session_name'), self.get('window_id')),
-            '-F%s' % ''.join(tmux_formats),     # output
-        ).stdout
-
-        # zip and map the results into the dict of formats used above
-        panes = [dict(zip(formats, pane.split('\t'))) for pane in panes]
-
-        # clear up empty dict
-        panes = [
-            dict((k, v) for k, v in pane.iteritems() if v) for pane in panes
-        ]
-
-        # filter by window_id
-        # this is causing errors with winderbuilder, the early window_id is
-        # returning as 0 when building sessions
-        # panes = [
-           # pane for pane in panes if pane['window_id'] == self.get('window_id')
-        # ]
-
-        # panes = [
-           # pane for pane in panes if pane['session_id'] == self.get('session_id')
-        # ]
-
-        new_panes = panes
-
-        if not self._panes:
-            for pane in new_panes:
-                logger.debug('adding pane_id %s for window_id %s' % (
-                    pane['pane_id'], pane['window_id']))
-                self._panes.append(Pane(window=self, **pane))
-            return self._panes
-
-        new = {pane['pane_id']: pane for pane in new_panes}
-        old = {pane.get('pane_id'): pane for pane in self._panes}
-
-        created = set(new.keys()) - set(old.keys()) or ()
-        deleted = set(old.keys()) - set(new.keys()) or ()
-        intersect = set(new.keys()).intersection(set(old.keys()))
-
-        diff = {id: dict(set(new[id].items()) - set(old[id].items()))
-                for id in intersect}
-
-        intersect = set(k for k, v in diff.iteritems() if v) or ()
-        diff = dict((k, v) for k, v in diff.iteritems() if v) or ()
-
-        if diff or created or deleted:
-            log_diff = "sync sessions for server:\n"
-        else:
-            log_diff = None
-        if diff and intersect:
-            log_diff += "diff %s for %s" % (diff, intersect)
-        if created:
-            log_diff += "created %s" % created
-        if deleted:
-            log_diff += "deleted %s" % deleted
-        if log_diff:
-            logger.debug(log_diff)
-
-        for p in self._panes:
-            # remove pane objects if deleted or out of session
-            if p.get('pane_id') in deleted or self.get('session_id') != p.get('session_id'):
-                logger.debug("removing %s" % p)
-                self._panes.remove(p)
-
-            if p.get('pane_id') in intersect and p.get('pane_id') in diff:
-                logger.debug('updating pane_id %s window_id %s' % (
-                    p.get('pane_id'), p.get('window_id')))
-                p.update(diff[p.get('pane_id')])
-
-            # create pane objects for non-existant pane_id's
-            for pane in [new[pane_id] for pane_id in created]:
-                logger.debug('adding pane_id %s window_id %s' % (
-                    pane['pane_id'], pane['window_id']))
-                self._panes.append(Pane(window=self, **pane))
-
-            return self._panes
     list_children = list_panes
+
+    @property
+    def panes(self):
+        return self.list_panes()
